// Copyright (C) 2010 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [mu,b,iter] = linalg_rayleighiteration ( A , mu , b , rtol , itmax , verbose )
    //   Computes approximated a pair of eigenvalue and eigenvector.
    //
    // Calling Sequence
    //   [mu,b,iter] = linalg_rayleighiteration ( A , mu0 , b0 , rtol , itmax , verbose )
    //
    // Parameters
    //   A : a n-by-n matrix of doubles, real, symetric
    //   mu0 : a 1-by-1 matrix of doubles, an estimated eigenvalue
    //   b0 : a n-by-1 matrix of doubles, an estimated eigenvector
    //   rtol : a 1-by-1 matrix of doubles, a relative tolerance
    //   itmax : a maximum number of iterations
    //   verbose : a 1-by-1 matrix of booleans, set to true to print convergence messages
    //   mu : a 1-by-1 matrix of doubles, an approximated eigenvalue
    //   b : a n-by-1 matrix of doubles, an approximated eigenvector
    //   iter : the number of iterations performed
    //
    // Description
    //   Uses Rayleigh iterations to find an approximate 
    //   eigenvalue and an approximate eigenvector.
    //
    //   The algorithm stops if norm(A*b-mu*b) < rtol*norm(mu*b).
    //
    //   Caution!
    //   The algorithm has problems when it comes closer to 
    //   the exact result, because an intermediate matrix 
    //   becomes more and more ill-conditionned.
    //   In this case, Scilab switches the backslash operator 
    //   from Gaussian elimination to least squares, which 
    //   makes the algorithms going nuts.
    //   Hence, we cannot use this algorithm with a relative 
    //   tolerance which is too small.
    //
    // Examples
    // // Example #1
    // A = [
    // 6 2 3
    // 2 5 1
    // 3 1 3
    // ];
    // b = [1 1 1]';
    // mu = 1;
    // tol = 1.e-4;
    // [mu,b,iter]=linalg_rayleighiteration ( A , mu , b , tol , 20 , %f )
    //
    // // Example #2
    // A = [
    // 1 2 3
    // 1 2 1
    // 3 2 1
    // ];
    // b = [1 1 1]';
    // mu = 200;
    // tol = 1.e-4;
    // [mu,b,iter]=linalg_rayleighiteration ( A , mu , b , tol , 100 , %f )
    //
    // // Example #3
    // // The initial eigenvalue is close to the 
    // // exact one.
    // // This makes Scilab switch to least squares and 
    // // prints a warning.
    // //
    // A = [
    // 1 2 3
    // 1 2 1
    // 3 2 1
    // ];
    // b = [1 1 1]';
    // mu = 0.7639320225002093067701;
    // tol = 1.e-4;
    // [mu,b,iter]=linalg_rayleighiteration ( A , mu , b , tol , 100 , %f )
    //
    // Authors
    // Copyright (C) 2010 - 2011 - Michael Baudin


    I = eye(A)
    iter = 1
    bprev = b
    Ab = A*b
    for iter = 1 : itmax
        if ( norm(Ab-mu*b)<rtol * norm(mu*b)) then
            break
        end
        C = A-mu*I
        x = C\b
        //x = linalg_gausspivotal ( C , b , %f )
        bprev = b
        b = x/norm(x)
        Ab = A*b
        mu = b'*Ab/(b'*b)
        if ( verbose ) then
        mprintf("iter=%d, mu=%s, b=%s\n",iter,string(mu),strcat(string(b)," "))
        end
    end
    //
    // Make sure that the first component of b is positive
    if ( b(1) <0 ) then
        b = -b
    end
endfunction

