// Copyright (C) 2008 - 2010 - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


function x = linalg_solvelu ( L , U , b , verbose )
  // Computes the solution of a linear equation, given its LU decomposition.
  // 
  // Calling Sequence
  //   x = linalg_solvelu ( L , U , b , verbose )
  //
  // Parameters
  // L : a n-by-n matrix of doubles, lower triangular.
  // U : a n-by-n matrix of doubles, upper triangular.
  // b : a n-by-1 matrix of doubles
  // verbose : a 1-by-1 matrix of boolean, set to true to display intermediate messages
  // x : a n-by-1 matrix of doubles
  //
  // Description
  //   Solve Ax=b with A=L*U and
  //   L lower triangular and U upper triangular.
  //   Effective algorithm with vectorization.
  //
  // Examples
  // L=[
  // 1 0 0 0 0
  // 1 1 0 0 0
  // 1 1 1 0 0
  // 1 1 1 1 0
  // 1 1 1 1 1
  // ];
  // U=[
  // 1 1 1 1 1
  // 0 1 1 1 1
  // 0 0 1 1 1
  // 0 0 0 1 1
  // 0 0 0 0 1
  // ];
  // b=[5;9;12;14;15];
  // xe=[1;1;1;1;1]
  // x=linalg_solvelu(L,U,b,%f)
  //
  // Authors
  // Copyright (C) 2008 - 2010 - Michael Baudin
  // Copyright (C) 2010 - DIGITEO - Michael Baudin
  //

  n=size(L,"r")
  y=zeros(n,1)
  // Forward loop
  for i=1:n
    y(i)=(b(i)-L(i,1:i-1)*y(1:i-1))/L(i,i)
  end
  if (verbose) then
    printf("After forward loop, y:\n");
    disp(y);
  end
  // Backward loop
  x=zeros(n,1)
  for i=n:-1:1
    x(i)=(y(i)-U(i,i+1:n)*x(i+1:n))/U(i,i)
  end
  if (verbose) then
    printf("After backward loop, x:\n");
    disp(x);
  end
endfunction


